package com.supervision.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supervision.model.AskTemplateQuestionLibrary;
import com.supervision.model.CommonDic;
import com.supervision.model.DiagnosisQaRecord;
import com.supervision.service.AskTemplateQuestionLibraryService;
import com.supervision.service.CommonDicService;
import com.supervision.service.DiagnosisQaRecordService;
import com.supervision.mapper.DiagnosisQaRecordMapper;
import com.supervision.vo.ask.DiagnosisQaRecordHistoryResVO;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
* @author flevance
* @description 针对表【vp_diagnosis_qa_record(诊断问答记录表)】的数据库操作Service实现
* @createDate 2023-10-20 17:19:21
*/
@Service
@RequiredArgsConstructor
public class DiagnosisQaRecordServiceImpl extends ServiceImpl<DiagnosisQaRecordMapper, DiagnosisQaRecord>
    implements DiagnosisQaRecordService{

    private final CommonDicService commonDicService;

    private final AskTemplateQuestionLibraryService askTemplateQuestionLibraryService;
    @Override
    public List<DiagnosisQaRecordHistoryResVO> queryByProcessId(String processId) {

        List<DiagnosisQaRecord> list = this.lambdaQuery().eq(DiagnosisQaRecord::getProcessId, processId).list();

        // 根据questionLibraryId找到对应的字典
        Set<String> questionIdList = list.stream().map(DiagnosisQaRecord::getQuestionLibraryId).filter(StrUtil::isNotBlank).collect(Collectors.toSet());
        List<AskTemplateQuestionLibrary> askTemplateQuestionLibraryList = askTemplateQuestionLibraryService.listByIds(questionIdList);
        Map<String, Long> questionLibraryMap = askTemplateQuestionLibraryList.stream().collect(Collectors.toMap(AskTemplateQuestionLibrary::getId, AskTemplateQuestionLibrary::getDictId));
        List<CommonDic> commonDicList = commonDicService.lambdaQuery().eq(CommonDic::getGroupCode, "AQT").list();
        Map<Long, CommonDic> dicMap = commonDicList.stream().collect(Collectors.toMap(CommonDic::getId, Function.identity()));
        List<DiagnosisQaRecordHistoryResVO> resVOS = BeanUtil.copyToList(list, DiagnosisQaRecordHistoryResVO.class);
        for (DiagnosisQaRecordHistoryResVO resVO : resVOS) {
            Long dictId = questionLibraryMap.get(resVO.getQuestionLibraryId());
            if (ObjectUtil.isNotEmpty(dictId)){
                CommonDic commonDic = dicMap.get(dictId);
                resVO.setCommonDic(commonDic);
            }
        }
        return resVOS;
    }
}




