package com.supervision.controller;


import com.supervision.model.DiagnosisPrimary;
import com.supervision.model.Disease;
import com.supervision.pojo.vo.DiagnosticBasisForPrimaryReqVO;
import com.supervision.pojo.vo.RecordForPrimaryChooseVO;
import com.supervision.pojo.vo.SaveDiagnosisPrimaryReqVO;
import com.supervision.vo.ask.DiagnosticBasisForPrimaryResVO;
import com.supervision.service.AskPrimaryService;
import com.supervision.vo.ask.DiagnosisPrimaryVO;
import com.supervision.vo.ask.ModifyPrimaryDiseaseInfoReqVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import java.util.List;

@Api(tags = "初步诊断")
@RestController
@RequestMapping("askPrimary")
@RequiredArgsConstructor
public class AskPrimaryController {

    private final AskPrimaryService askPrimaryService;

    @ApiOperation("查询初步诊断列表")
    @GetMapping("queryAskPrimaryList")
    public List<DiagnosisPrimaryVO> queryAskPrimaryList(String processId){
        return askPrimaryService.queryAskPrimaryList(processId);
    }

    @ApiOperation("获取初步诊断可选的下拉列表")
    @GetMapping("queryPrimaryCanChooseList")
    public List<Disease> queryPrimaryCanChooseList(){
        return askPrimaryService.queryPrimaryCanChooseList();
    }

    @ApiOperation("修改初步诊断病情依据")
    @PostMapping("modifyPrimaryDiseaseInfo")
    public void modifyPrimaryDiseaseInfo(@RequestBody ModifyPrimaryDiseaseInfoReqVO reqVO){
        askPrimaryService.modifyPrimaryDiseaseInfo(reqVO);
    }

    @ApiOperation("新增初步诊断")
    @PostMapping("savePrimary")
    public void savePrimary(@RequestBody SaveDiagnosisPrimaryReqVO reqVO){
        askPrimaryService.savePrimary(reqVO);
    }

    @ApiOperation("查询选择初步诊断关联的问诊记录")
    @GetMapping("queryRecordForPrimaryChoose")
    public RecordForPrimaryChooseVO queryRecordForPrimaryChoose(@NotBlank(message = "流程ID不能为空")String processId){
        return askPrimaryService.queryRecordForPrimaryChoose(processId);
    }

    @ApiOperation("确认诊断时,修改初步诊断的信息")
    @PostMapping("updatePrimaryByConfirm")
    public void updatePrimaryByConfirm(@RequestBody DiagnosisPrimary reqVO){
        askPrimaryService.updatePrimaryByConfirm(reqVO);
    }

    @ApiOperation("删除初步诊断")
    @GetMapping("deletePrimary")
    public void deletePrimary(String id){
        askPrimaryService.deletePrimary(id);
    }

    @ApiOperation("查询初步诊断需要填写的诊断依据")
    @GetMapping("queryDiagnosticBasisListForPrimary")
    public List<DiagnosticBasisForPrimaryResVO> queryDiagnosticBasisListForPrimary(@NotBlank(message = "流程ID不能为空") String processId){
        return askPrimaryService.queryDiagnosticBasisListForPrimary(processId);
    }

    @ApiOperation("确认诊断-保存诊断依据")
    @PostMapping("saveDiagnosticBasis")
    public void saveDiagnosticBasis(@RequestBody List<DiagnosticBasisForPrimaryReqVO> reqVO){
        askPrimaryService.saveDiagnosticBasis(reqVO);
    }
}
